#!/usr/local/bin/perl -w

use File::Compare;

sub test;

# test for output then exit completely none is found
print "Testing that results exist...\t\t\t";
test( not -t STDIN ) or die;

@input = <STDIN>;

# test if output is correctly formatted
print "Testing that output is correctly formatted...\t";
$valid = 1;
for( $i = 0; $i < $#input; $i += 5 ) {
	# not valid if only part of a recipe is there	
	if( $i+3 > $#input ) { 
		$valid = 0;
		last; 
	}

	# check that the title is trimmed
	if( not $input[$i] =~ m/^\S.*\S$/ ) { 
		$valid = 0;
		last; 
	}
	
	# check that the decription is trimmed
	if( not $input[$i+1] =~ m/^\S.*\S$/ ) { 
		$valid = 0;
		last; 
	}

	# check that ingredients are tab delimited and there's at least one
	if( not $input[$i+2] =~ m/^(\S.*\t)+$/ ) { 
		$valid = 0;
		last; 
	}

	# check that directions are tab delimited, start with a number, and there's at least one
	if( not $input[$i+3] =~ m/^([0-9]\.\s\S.*\t)+$/ ) { 
		$valid = 0;
		last; 
	}

	# make sure there is a blank line before the next record
	if( $input[$i+4] =~ m/^.+$/ ) { 
		$valid = 0;
		last; 
	}
}
test( $valid );

# test if output is the same as the given file
if( $#ARGV >= 0 ) {
	print "Testing Result Correctness...\t\t\t";

	if( not -e $ARGV[0] ) {
		print "$ARGV[0] does not exist!\n";
		die;
	} 
	
	open CORRECT, $ARGV[0];
	@output = <CORRECT>;

	$same = 1;

	if( $#input != $#output ) {
		$same = 0;
	}
	else {
		for( $i = 0; $i < $#input; $i++ ) {
			if( $input[$i] ne $output[$i] ) {
				$same = 0;
				last;
			}
		}
	}

	close CORRECT;

	test( $same );
}

sub test {
	$result = shift;

	if( $result ) 	{ print "Passed!\n"; }
	else 		{ print "Failed!\n"; }

	return $result;
}
