<?php

class Search_Controller extends AbstractController
{
	function index()
	{
		$this->load->model('Recipe_Model', 'recipe');
		$this->load->model('Special_Model', 'special');
		$this->load->model('Favorites_Model', 'favorite');

		$query = Url::get('q'); // search query
		$id = Url::get('id'); // if a specific recipe id or special id is given.
		$type = Url::get('type');
		$recipe_search = $type == 'recipe' || $type == 'both';
		$special_search = $type == 'special' || $type == 'both';

		$zipCode = Url::get('zipCode');
		$zipCode = (strncasecmp($zipCode, 'zipcode', 7) == 0) ? null : $zipCode;

		// preserve zipcode in session (might need it for subsequent recipe searches)
		if ($zipCode)
			$_SESSION['zipCode'] = $zipCode;

		// use the user's zip code if absolutely necessary.
		if (!$zipCode && $this->user->isLoggedIn())
			$zipCode = $this->user->zipCode();

		// use the session zip code as a last resort.
		if (!$zipCode && isset($_SESSION['zipCode']))
			$zipCode = $_SESSION['zipCode'];

		// set the user's zip code for future use.
		if ($this->user->isLoggedIn() && $zipCode)
			$this->user->zipCode($zipCode);

		$this->view->assign('user', $this->user->getArray());
		$this->view->assign('recipe_search', $recipe_search);
		$this->view->assign('special_search', $special_search);
		$this->view->assign('view_recipe', $id && $recipe_search);
		$this->view->assign('view_special', $id && $special_search);

		// *************************************
		// Recipe Search
		if ($recipe_search) {
			$recipes = array();
			if ($id) {
				$recipe = $this->recipe->getRecipeById($id);
				$recipes[] = $recipe;
			} else {
				// recipe results.
				$recipes = $this->recipe->getRecipesByName($query);
				$recipes = $this->_mergeAssoc($recipes, $this->recipe->getRecipesByIngredient($query), 'id');
			}

			// get a list of associated specials for recipes.
			$specials = array();
			foreach ($recipes as $r) {
				$s = $this->special->getSpecialsForRecipe($r['id'], $zipCode);
				$specials[$r['id']] = $s;
			}

			$this->view->assign('recipes', $recipes);
			$this->view->assign('associated_specials', $specials);
		}

		// *************************************
		// Special Search
		if ($special_search) {
			$specials = array();
			if ($id) {
				$special = $this->special->getSpecialById($id);
				$specials[] = $special;
			} else {
				// special results. (search both by ingredient and special title and special description)
				$specials = $this->special->getSpecialsByName($query);
				$specials = $this->_mergeAssoc($specials, $this->special->getSpecialsByIngredient($query), 'id');
				$specials = $this->special->filterByZipCode($specials, $zipCode);
			}

			// get a list of associated recipes for specials.
			$recipes = array();
			foreach ($specials as $s) {
				$r = $this->recipe->getRecipesForSpecial($s['id']);
				$recipes[$s['id']] = $r;
			}

			$this->view->assign('specials', $specials);
			$this->view->assign('associated_recipes', $recipes);
		}

		// *************************************
		// User Favorite Foods
		if ($this->user->isLoggedIn()) {
			$special_ids = $this->favorite->getSavedSpecialIds($this->user->id());
			$favorite_specials = $this->favorite->getSavedSpecials($this->user->id(), $this->special);
			$favorite_specials = !empty($special_ids)
				? array_combine($special_ids, $favorite_specials)
				: array();

			$recipe_ids = $this->favorite->getSavedRecipeIds($this->user->id());
			$favorite_recipes = $this->favorite->getSavedRecipes($this->user->id(), $this->recipe);
			$favorite_recipes = !empty($recipe_ids)
				? array_combine($recipe_ids, $favorite_recipes)
				: array();

			$this->view->assign('favorite', array(
				'specials' => $favorite_specials,
				'recipes' => $favorite_recipes,
				'foods' => $this->favorite->getSavedFoods($this->user->id()),
			));
		}

		$this->view->display('search.html');
	}

	function _mergeAssoc($array1, $array2, $key) {
		$result = array();

		foreach ($array1 as $a1) {
			if (!$this->_hasKeyVal($array2, $key, $a1[$key])) {
				$result[] = $a1;
			}
		}

		return array_merge($result, $array2);
	}

	function _hasKeyVal($array, $key, $value) {
		foreach ($array as $a) {
			if ($a[$key] == $value) {
				return true;
			}
		}

		return false;
	}
}

?>