<?php

class Special_Model extends AbstractModel
{
	function filterByZipCode($specials, $zipCode)
	{
		$zipCode = intval(preg_replace('([^0-9])', '', $zipCode));
		$result = array();
		foreach ($specials as $special) {
			if (intval(preg_replace('([^0-9])', '', $special['store']['zipCode'])) == $zipCode) {
				$result[] = $special;
			}
		}

		return $result;
	}

	function getSpecialsByName($query)
	{
		$query = $this->db->getLike($query);

		$specials = $this->db->queryAll('
			select s.id, s.title, s.description, s.price
			from special s
			where
				(s.title like ? or s.description like ?)',
			array($query, $query));

		$specials = $this->_joinStoreInfo($specials);
		return $specials;
	}

	function getSpecialsByIngredient($query)
	{
		$query = $this->db->getLike($query);

		$specials = $this->db->queryAll('
			select s.id, s.title, s.description, s.price
			from special s, specialingredient si, ingredient i
			where si.specialId = s.id
				and i.id = si.ingredientId
				and i.name LIKE ?', array($query));

		$specials = $this->_joinStoreInfo($specials);
		return $specials;
	}

	function getSpecialsByArea($zipCode)
	{
		$zipCode = $this->db->getLike($zipCode);
		$specials = $this->db->queryAll('
			select s.id, s.title, s.description, s.price
			from special s, storeLocation sl
			where s.storeLocationId = sl.id
				and sl.zipCode like ?', array($zipCode));
		$specials = $this->_joinStoreInfo($specials);
		return $specials;
	}

	function getSpecialById($id)
	{
		$special = $this->db->queryRow('
			select id, title, description, price
			from special
			where id = ?', array($id));

		if ($special) {
			$special['store'] = $this->_getStoreInfo($special);
			return $special;
		}

		return null;
	}

	function getSpecialsForRecipe($recipeId, $zipCode = null)
	{
		if ($zipCode) {
			$specials = $this->db->queryAll('
				select distinct s.id, s.title, s.description, s.price
				from special s, specialingredient si, recipeingredient ri, storelocation sl
				where ri.ingredientId = si.ingredientId
					and s.id = si.specialId
					and s.storeLocationId = sl.id
					and ri.recipeId = ?
					and sl.zipCode like ?', array($recipeId, $zipCode));
		} else {
			$specials = $this->db->queryAll('
				select distinct s.id, s.title, s.description, s.price
				from special s, specialingredient si, recipeingredient ri
				where ri.ingredientId = si.ingredientId and s.id = si.specialId
					and ri.recipeId = ?', array($recipeId));
		}

		$specials = $this->_joinStoreInfo($specials);
		return $specials;
	}

	function _joinStoreInfo($specials) {
		$result = array();
		foreach ($specials as $special) {
			$special['store'] = $this->_getStoreInfo($special);
			$result[] = $special;
		}

		return $result;
	}

	function _getStoreInfo($special) {
		return $this->db->queryRow('
			select sl.id as storeLocationId, sl.storeChainId, sl.address, sl.city, sl.state, sl.zipCode, sl.phone,
				sc.name, sc.websiteUrl
			from special s, storelocation sl, storechain sc
			where s.storeLocationId = sl.id and sl.storeChainId = sc.id
				and s.id = ?', array($special['id']));
	}
}

?>